@@ -233,13 +233,10 @@ SEXP TikZ_StartDevice ( SEXP args ){
 /*
  * This function is responsible for initializing device parameters
  * contained in the variable deviceInfo. It returns a true or false
- * value depending on the success of initialization operations. The
- * static keyword means this function can only be seen by other functions
- * in this file. A better choice for the keyword might have been something
- * like "private"...
+ * value depending on the success of initialization operations.
 */
 
-static Rboolean TikZ_Setup(
+Rboolean TikZ_Setup(
   pDevDesc deviceInfo,
   const char *fileName,
   double width, double height, Rboolean onefile,
@@ -419,6 +416,10 @@ static Rboolean TikZ_Setup(
   deviceInfo->haveLocator = 1;       /* 1 = no, 2 = yes */
 #endif
 
+#if R_GE_version >= 13
+  deviceInfo->deviceVersion = R_GE_definitions;
+#endif
+
   /*
    * Initialize device parameters. These concern properties such as the
    * plotting canvas size, the initial foreground and background colors and
@@ -502,6 +503,15 @@ static Rboolean TikZ_Setup(
   deviceInfo->raster = TikZ_Raster;
   deviceInfo->cap = TikZ_Cap;
 
+#if R_GE_version >= 13
+  deviceInfo->setPattern      = TikZ_setPattern;
+  deviceInfo->releasePattern  = TikZ_releasePattern;
+  deviceInfo->setClipPath     = TikZ_setClipPath;
+  deviceInfo->releaseClipPath = TikZ_releaseClipPath;
+  deviceInfo->setMask         = TikZ_setMask;
+  deviceInfo->releaseMask     = TikZ_releaseMask;
+#endif
+
   /* Dummy routines. These are mainly used by GUI graphics devices. */
   deviceInfo->activate = TikZ_Activate;
   deviceInfo->deactivate = TikZ_Deactivate;
@@ -520,7 +530,7 @@ static Rboolean TikZ_Setup(
   return TRUE;
 }
 
-static void TikZ_WriteColorDefinition( tikzDevDesc *tikzInfo, void (*printOut)(tikzDevDesc *tikzInfo, const char *format, ...), int color, const char* colorname, const char* colorstr )
+void TikZ_WriteColorDefinition( tikzDevDesc *tikzInfo, void (*printOut)(tikzDevDesc *tikzInfo, const char *format, ...), int color, const char* colorname, const char* colorstr )
 {
 
   debug_print_empty();
@@ -543,7 +553,7 @@ static void TikZ_WriteColorDefinition( tikzDevDesc *tikzInfo, void (*printOut)(t
       R_BLUE(color));
 }
 
-static void TikZ_WriteColorDefinitions( tikzDevDesc *tikzInfo )
+void TikZ_WriteColorDefinitions( tikzDevDesc *tikzInfo )
 {
 
   debug_print_empty();
@@ -561,7 +571,7 @@ static void TikZ_WriteColorDefinitions( tikzDevDesc *tikzInfo )
   }
 }
 
-static void TikZ_WriteColorFile(tikzDevDesc *tikzInfo)
+void TikZ_WriteColorFile(tikzDevDesc *tikzInfo)
 {
 
   debug_print_empty();
@@ -600,7 +610,7 @@ static void TikZ_WriteColorFile(tikzDevDesc *tikzInfo)
  * - Clip
  * - Size
  */
-static Rboolean TikZ_Open( pDevDesc deviceInfo )
+Rboolean TikZ_Open( pDevDesc deviceInfo )
 {
 
   debug_print_empty();
@@ -613,7 +623,10 @@ static Rboolean TikZ_Open( pDevDesc deviceInfo )
 
   /* If creating multiple files, add the page number to the filename. */
   if ( !tikzInfo->onefile )
-    sprintf(tikzInfo->outFileName, tikzInfo->originalFileName, tikzInfo->pageNum);
+    snprintf(tikzInfo->outFileName,
+             strlen(tikzInfo->originalColorFileName)+floor(log10(tikzInfo->pageNum))+1,
+             tikzInfo->originalFileName,
+             tikzInfo->pageNum);
 
   if( strlen(tikzInfo->originalColorFileName) > 0 )
     {
@@ -653,7 +666,7 @@ static Rboolean TikZ_Open( pDevDesc deviceInfo )
   return TRUE;
 }
 
-static void TikZ_Close( pDevDesc deviceInfo)
+void TikZ_Close( pDevDesc deviceInfo)
 {
 
   debug_print_empty();
@@ -684,7 +697,7 @@ static void TikZ_Close( pDevDesc deviceInfo)
       tikzInfo->stringWidthCalls);
 
   /* Close the file and destroy the tikzInfo structure. */
-  if(tikzInfo->console == FALSE)
+  if(tikzInfo->console == FALSE && tikzInfo->outputFile)
   {
     fclose(tikzInfo->outputFile);
     tikzInfo->outputFile = NULL;
@@ -710,7 +723,7 @@ static void TikZ_Close( pDevDesc deviceInfo)
 
 }
 
-static void TikZ_NewPage( const pGEcontext plotParams, pDevDesc deviceInfo )
+void TikZ_NewPage( const pGEcontext plotParams, pDevDesc deviceInfo )
 {
 
   debug_print_empty();
@@ -755,7 +768,7 @@ static void TikZ_NewPage( const pGEcontext plotParams, pDevDesc deviceInfo )
   tikzInfo->pageState = TIKZ_START_PAGE;
 }
 
-static void TikZ_Clip( double x0, double x1,
+void TikZ_Clip( double x0, double x1,
     double y0, double y1, pDevDesc deviceInfo )
 {
 
@@ -788,7 +801,7 @@ static void TikZ_Clip( double x0, double x1,
   tikzInfo->clipState = TIKZ_START_CLIP;
 }
 
-static void TikZ_Size( double *left, double *right,
+void TikZ_Size( double *left, double *right,
     double *bottom, double *top, pDevDesc deviceInfo){
 
   debug_print_empty();
@@ -820,7 +833,7 @@ static void TikZ_Size( double *left, double *right,
  * is most obvious when plot legends are created.
  *
 */
-static void TikZ_MetricInfo(int c, const pGEcontext plotParams,
+void TikZ_MetricInfo(int c, const pGEcontext plotParams,
     double *ascent, double *descent, double *width, pDevDesc deviceInfo ){
 
   debug_print_empty();
@@ -945,7 +958,7 @@ static void TikZ_MetricInfo(int c, const pGEcontext plotParams,
  * better.
  *
 */
-static double TikZ_StrWidth( const char *str,
+double TikZ_StrWidth( const char *str,
     const pGEcontext plotParams, pDevDesc deviceInfo ){
 
   debug_print_empty();
@@ -1142,7 +1155,7 @@ static double TikZ_StrWidth( const char *str,
  *
  * The rotation value is given in degrees.
 */
-static void TikZ_Text( double x, double y, const char *str,
+void TikZ_Text( double x, double y, const char *str,
     double rot, double hadj, const pGEcontext plotParams,
     pDevDesc deviceInfo){
 
@@ -1251,7 +1264,7 @@ static void TikZ_Text( double x, double y, const char *str,
 }
 
 
-static void TikZ_Circle( double x, double y, double r,
+void TikZ_Circle( double x, double y, double r,
     const pGEcontext plotParams, pDevDesc deviceInfo){
 
   debug_print_empty();
@@ -1278,7 +1291,7 @@ static void TikZ_Circle( double x, double y, double r,
     x,y,r);
 }
 
-static void TikZ_Rectangle( double x0, double y0,
+void TikZ_Rectangle( double x0, double y0,
     double x1, double y1, const pGEcontext plotParams, pDevDesc deviceInfo){
 
   debug_print_empty();
@@ -1308,7 +1321,7 @@ static void TikZ_Rectangle( double x0, double y0,
 }
 
 
-static void TikZ_Line( double x1, double y1,
+void TikZ_Line( double x1, double y1,
     double x2, double y2, const pGEcontext plotParams, pDevDesc deviceInfo){
 
   debug_print_empty();
@@ -1337,7 +1350,7 @@ static void TikZ_Line( double x1, double y1,
 }
 
 
-static void TikZ_Polyline( int n, double *x, double *y,
+void TikZ_Polyline( int n, double *x, double *y,
     pGEcontext plotParams, pDevDesc deviceInfo ){
 
   debug_print_empty();
@@ -1403,7 +1416,7 @@ static void TikZ_Polyline( int n, double *x, double *y,
 
 }
 
-static void TikZ_Polygon( int n, double *x, double *y,
+void TikZ_Polygon( int n, double *x, double *y,
     pGEcontext plotParams, pDevDesc deviceInfo ){
 
   debug_print_empty();
@@ -1456,7 +1469,7 @@ static void TikZ_Polygon( int n, double *x, double *y,
 }
 
 
-static void
+void
 TikZ_Path( double *x, double *y,
   int npoly, int *nper,
   Rboolean winding,
@@ -1535,7 +1548,7 @@ TikZ_Path( double *x, double *y,
  * say PNG, and then dropping a node in the TikZ output that contains
  * an \includegraphics directive.
 */
-static void TikZ_Raster(
+void TikZ_Raster(
   unsigned int *raster,
   int w, int h,
   double x, double y,
@@ -1674,7 +1687,7 @@ static void TikZ_Raster(
  * Argument for implementation: could be useful for "previewing" the
  * current* state of the tikzDevice output.
 */
-static SEXP TikZ_Cap( pDevDesc deviceInfo ){
+SEXP TikZ_Cap( pDevDesc deviceInfo ){
 
   debug_print_empty();
 
@@ -1692,15 +1705,15 @@ static SEXP TikZ_Cap( pDevDesc deviceInfo ){
  * to a mouse click. For devices plotting to files these functions can be left
  * as dummy routines.
 */
-static void TikZ_Activate( pDevDesc deviceInfo ){
+void TikZ_Activate( pDevDesc deviceInfo ){
   debug_print_empty();
 }
 
-static void TikZ_Deactivate( pDevDesc deviceInfo ){
+void TikZ_Deactivate( pDevDesc deviceInfo ){
   debug_print_empty();
 }
 
-static Rboolean TikZ_Locator( double *x, double *y, pDevDesc deviceInfo ){
+Rboolean TikZ_Locator( double *x, double *y, pDevDesc deviceInfo ){
 
   debug_print_empty();
 
@@ -1713,11 +1726,29 @@ static Rboolean TikZ_Locator( double *x, double *y, pDevDesc deviceInfo ){
  * so this function is a dummy routine. Conciveably this function could be
  * used to wrap TikZ graphics in \begin{scope} and \end{scope} directives.
 */
-static void TikZ_Mode( int mode, pDevDesc deviceInfo ){
+void TikZ_Mode( int mode, pDevDesc deviceInfo ){
 
   debug_print_empty();
 }
 
+SEXP TikZ_setPattern(SEXP pattern, pDevDesc dd) {
+    return R_NilValue;
+}
+
+void TikZ_releasePattern(SEXP ref, pDevDesc dd) {}
+
+SEXP TikZ_setClipPath(SEXP path, SEXP ref, pDevDesc dd) {
+    return R_NilValue;
+}
+
+void TikZ_releaseClipPath(SEXP ref, pDevDesc dd) {}
+
+SEXP TikZ_setMask(SEXP path, SEXP ref, pDevDesc dd) {
+    return R_NilValue;
+}
+
+void TikZ_releaseMask(SEXP ref, pDevDesc dd) {}
+
 /*==============================================================================
 
                           End Core Graphics Routines
@@ -1736,7 +1767,7 @@ static void TikZ_Mode( int mode, pDevDesc deviceInfo ){
  * determine if a given plotting opertion will result in a visible fill or
  * stroke.
  */
-static TikZ_DrawOps TikZ_GetDrawOps(pGEcontext plotParams)
+TikZ_DrawOps TikZ_GetDrawOps(pGEcontext plotParams)
 {
 
   debug_print_empty();
@@ -1758,7 +1789,7 @@ static TikZ_DrawOps TikZ_GetDrawOps(pGEcontext plotParams)
   return ops;
 }
 
-static Rboolean TikZ_CheckColor(tikzDevDesc *tikzInfo, int color)
+Rboolean TikZ_CheckColor(tikzDevDesc *tikzInfo, int color)
 {
 
   debug_print_empty();
@@ -1774,7 +1805,7 @@ static Rboolean TikZ_CheckColor(tikzDevDesc *tikzInfo, int color)
   return FALSE;
 }
 
-static Rboolean TikZ_CheckAndAddColor(tikzDevDesc *tikzInfo, int color)
+Rboolean TikZ_CheckAndAddColor(tikzDevDesc *tikzInfo, int color)
 {
 
   debug_print_empty();
@@ -1803,7 +1834,7 @@ static Rboolean TikZ_CheckAndAddColor(tikzDevDesc *tikzInfo, int color)
   return colorfound;
 }
 
-static void TikZ_DefineDrawColor(tikzDevDesc *tikzInfo, int color, TikZ_DrawOps ops)
+void TikZ_DefineDrawColor(tikzDevDesc *tikzInfo, int color, TikZ_DrawOps ops)
 {
 
   debug_print_empty();
@@ -1840,7 +1871,7 @@ static void TikZ_DefineDrawColor(tikzDevDesc *tikzInfo, int color, TikZ_DrawOps
 
 }
 
-static void TikZ_DefineColors(pGEcontext plotParams, pDevDesc deviceInfo, TikZ_DrawOps ops)
+void TikZ_DefineColors(pGEcontext plotParams, pDevDesc deviceInfo, TikZ_DrawOps ops)
 {
 
   debug_print_empty();
@@ -1872,7 +1903,7 @@ static void TikZ_DefineColors(pGEcontext plotParams, pDevDesc deviceInfo, TikZ_D
  * be added after the call to `TikZ_WriteDrawOptions` and should remember to
  * bring their own commas.
  */
-static void TikZ_WriteDrawOptions(const pGEcontext plotParams, pDevDesc deviceInfo,
+void TikZ_WriteDrawOptions(const pGEcontext plotParams, pDevDesc deviceInfo,
     TikZ_DrawOps ops)
 {
 
@@ -1904,7 +1935,7 @@ static void TikZ_WriteDrawOptions(const pGEcontext plotParams, pDevDesc deviceIn
 
 }
 
-static void TikZ_WriteLineStyle(pGEcontext plotParams, tikzDevDesc *tikzInfo)
+void TikZ_WriteLineStyle(pGEcontext plotParams, tikzDevDesc *tikzInfo)
 {
 
   debug_print_empty();
@@ -1991,7 +2022,7 @@ static void TikZ_WriteLineStyle(pGEcontext plotParams, tikzDevDesc *tikzInfo)
  * base font size. The ratio is then further scaled by the value of
  * the character expansion factor cex.
 */
-static double
+double
 ScaleFont( const pGEcontext plotParams, pDevDesc deviceInfo ){
 
   debug_print_empty();
@@ -2103,7 +2134,7 @@ SEXP TikZ_EvalWithoutInterrupts(SEXP expr, SEXP envir){
 
 ==============================================================================*/
 
-static void printOutput(tikzDevDesc *tikzInfo, const char *format, ...){
+void printOutput(tikzDevDesc *tikzInfo, const char *format, ...){
 
   va_list(ap);
   va_start(ap, format);
@@ -2117,7 +2148,7 @@ static void printOutput(tikzDevDesc *tikzInfo, const char *format, ...){
 
 }
 
-static void printColorOutput(tikzDevDesc *tikzInfo, const char *format, ...){
+void printColorOutput(tikzDevDesc *tikzInfo, const char *format, ...){
 
   va_list(ap);
   va_start(ap, format);
@@ -2140,7 +2171,7 @@ static void printColorOutput(tikzDevDesc *tikzInfo, const char *format, ...){
  *   - The date on which the graphic was created.
  *
 */
-static void Print_TikZ_Header( tikzDevDesc *tikzInfo ){
+void Print_TikZ_Header( tikzDevDesc *tikzInfo ){
 
   debug_print_empty();
 
@@ -2180,7 +2211,7 @@ static void Print_TikZ_Header( tikzDevDesc *tikzInfo ){
 
 }
 
-static char *Sanitize(const char *str){
+char *Sanitize(const char *str){
 
   debug_print_empty();
 
@@ -2238,7 +2269,7 @@ static char *Sanitize(const char *str){
 }
 
 #if 0
-static Rboolean contains_multibyte_chars(const char *str){
+Rboolean contains_multibyte_chars(const char *str){
   /*
    * Recover package namespace as the multibyte check function
    * is not exported
@@ -2279,7 +2310,7 @@ static Rboolean contains_multibyte_chars(const char *str){
  * and TikZ, any combination of device and user dimensions could
  * theoretically be supported.
 */
-static double dim2dev( double length ){
+double dim2dev( double length ){
   return length*72.27;
 }
 
@@ -2290,7 +2321,7 @@ static double dim2dev( double length ){
  * this function allows the creation of new pages or clipping scopes to be
  * deferred until there is actually output to place in the environment.
  */
-static void TikZ_CheckState(pDevDesc deviceInfo)
+void TikZ_CheckState(pDevDesc deviceInfo)
 {
   tikzDevDesc *tikzInfo = (tikzDevDesc *) deviceInfo->deviceSpecific;
 
@@ -2306,7 +2337,7 @@ static void TikZ_CheckState(pDevDesc deviceInfo)
      */
     if( !tikzInfo->onefile )
       if( !TikZ_Open(deviceInfo) )
-        error("Unable to open output file: %s", tikzInfo->outputFile);
+        error("Unable to open output file: %s", tikzInfo->outFileName);
 
     if ( tikzInfo->debug == TRUE )
       printOutput(tikzInfo,
@@ -2362,11 +2393,11 @@ static void TikZ_CheckState(pDevDesc deviceInfo)
 
 }
 
-static char *calloc_strcpy(const char *str){
+char *calloc_strcpy(const char *str){
   return calloc_x_strcpy(str, 0);
 }
 
-static char *calloc_x_strcpy(const char *str, size_t extra){
+char *calloc_x_strcpy(const char *str, size_t extra){
   char *ret;
 
   ret = calloc_x_strlen(str, extra);
@@ -2374,15 +2405,15 @@ static char *calloc_x_strcpy(const char *str, size_t extra){
   return ret;
 }
 
-static char *calloc_x_strlen(const char *str, size_t extra){
+char *calloc_x_strlen(const char *str, size_t extra){
   return (char*) calloc(strlen(str) + 1 + extra, sizeof(char));
 }
 
-static void const_free(const void *ptr){
+void const_free(const void *ptr){
   free((void*)ptr);
 }
 
-static void strlcpy_(char *dst, const char* src, size_t n){
+void strlcpy_(char *dst, const char* src, size_t n){
   if (n <= 0) return;
   strncpy(dst, src, n - 1);
   dst[n - 1] = '\0';
